/***********************************************************************
 Demo for TORNADO-P6x with T/SDAS-ADDA4/16Q/200K AD/DA DCM.
 Copyright (C) MicroLAB(tm) Systems Ltd, 1999-2000

 description:
   1. This demo provides examples for programming and evaluation the DCM.
   2. Demo assumes that SIOX TM1 line is used for DCM mode control
      (J1 jumper should be in 2-3 position)

*************************************************************************/



#include <malloc.h>
#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <mlu.h>
#include <ips_ut.h>
#include <tp6x_h.h>
#include "ht6comm.i"

long	max_scan_len=0;
long   scan_len,   // epoch length in samples
       cmnd;       // amplitude

long	i;

int	tmp,ex_status,adc_scan_flag=0;
char	s[80],ch;
int	dat_type=4; // def for INT type (number of bytes per word)

MLU_TBDD TB;  // time bell descriptor


#define CLEAR_CURRENT_LINE  printf("\r%79s\r"," ");



struct MCD_DD  *MCD;     // MCD pointer

long    *D;               // master data buffer array

long   MADC_SADDR,
       MDAC_SADDR;

#define	_SXADDA4DCM_MODE_ASYNC	0
#define	_SXADDA4DCM_MODE_SYNC	1

#define	_SXADDA4DCM_FSSEL_FSG	0
#define	_SXADDA4DCM_FSSEL_EXT	1
#define	_SXADDA4DCM_FSSEL_SXTM	2
#define	_SXADDA4DCM_FSSEL_FSX	3

#define	_SXADDA4DCM_ADWFMT_64	0
#define	_SXADDA4DCM_ADWFMT_32	1
#define	_SXADDA4DCM_ADWFMT_16	2

#define	_SXADDA4DCM_CLKSEL_INT	0
#define	_SXADDA4DCM_CLKSEL_CLKX	1

#define	_C6X_FPER_SOURCE_CLK	15000000L // CLKX clock as source for C6x/FPER
#define	_C6X_FPER_MAX_COUNT	4095	  // maximum FPER count value


			// input dialog vars
long dcm_mode=_SXADDA4DCM_MODE_ASYNC,  	// mode=ASYNC
     fs_sel=_SXADDA4DCM_FSSEL_FSG,	// Fs selector = FSG
     fs_entered = 10000,		// sampling freq = 10 kHz
     fs,
     ad_chan=0xf,			// enable all A/D channels
     da_chan=0xf,                       // enable all D/A channels
     ad_chan_total=4,
     da_chan_total=4,
     synclk_sel=_SXADDA4DCM_CLKSEL_INT;// SYNCHRO-CLK = INTERNAL

int  kbd_cmnd;			// keyboard command
int  prev_kbd_adc_record_cmnd=0;// flag to detect enable for KBD_DAC_REPLAY_CMND

unsigned dev_id;


//--- enter 4-channel list

#define CH_TOTAL_MAX	4

unsigned _get_ch_mask (unsigned ch)
{ int i;
  unsigned ch_mask;

    for(i=0,ch_mask=1;i<CH_TOTAL_MAX;i++)
     { if(i==ch) return(ch_mask);
       ch_mask = ch_mask<<1;
     }
    return(0xffff);
}

int enter_channel_list (char *ch_pool, long *ch_array, long *ch_total)
{  unsigned channels, ch_history[5], ch_mask, ch_char;
   int i,n;

      channels = *ch_array & 0xf;

      printf("%s",ch_pool);

// print default channels if any
     for (i=0,n=0;i<CH_TOTAL_MAX;i++)
	{ if((channels & _get_ch_mask(i))!=0)
	    { printf("%d",i);
              ch_history[n] = i;
	      n++;
	    }
	}

    loop:
      switch(ch_char=read_char())
       {
         case '0':
         case '1':
         case '2':
         case '3':
		ch_mask = _get_ch_mask(ch_char-'0');

	    chan_check:
		if(n>=CH_TOTAL_MAX)	// check max 4 channels entered
					// wait for ENTER or BACKSPACE
	            { beep_err();
		      goto loop;
		    }
					// check for new channel entered
					// return if channel already entered
		if( (channels & ch_mask)!=0)
	            { beep_err();
		      goto loop;
		    }

		printf("%c",ch_char);	// print entered channel
                channels |= ch_mask;
		ch_history[n] = (ch_char-'0');
                n++;
		goto loop;


	 case KEY_ENTER:		// terminate
		    { printf("\n");
                      break;
		    }

	 case KEY_BACKSP:
		if (n>0)
                 { printf("\010 \010");
		   channels &= ~_get_ch_mask(ch_history[--n]);
                 }
                  else
		 {
		     beep_err();
		 }
                goto loop;


	 case KEY_ESC:
		      printf("<aborted>\n");
		      return(1);

	default:
		beep_err();
		goto loop;
       }

      *ch_array = channels;
      if (ch_total!=NULL) *ch_total = n;
      return(0);
}


#define KBD_DAC_SINE_CMND 	(kbd_cmnd=='0')
#define KBD_ADC_RECORD_CMND 	(kbd_cmnd=='1')
#define KBD_DAC_REPLAY_CMND 	(kbd_cmnd=='2')
#define KBD_ADC_TO_DAC_CMND 	(kbd_cmnd=='3')

inp_param()
{
  int i,n;
  unsigned ch_mask;

      printf("operation mode (0-async, 1-sync)                    : ");
      dcm_mode=getlong(0L,1L,dcm_mode,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK)
       {
         aborted:
	  printf(" <aborted>\n");
	  return(1);
       }

      printf("synchro-clock source (0-int, 1-CLKX)                : ");
      synclk_sel=getlong(0L,1L,synclk_sel,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

	//!! note: FSX is enabled only in ASYNC mode
	//         and in case SYNCHRO-CLK is derived fm SIOX/CLKX (C6x specific)
      printf("sampling freq source (0-FSG, 1-Ext, 2-SX/TM0, 3-FSX): ");
      fs_sel=getlong(0L,
      		     (dcm_mode==0 && synclk_sel==1)? 3L:2L,
      	             fs_sel,CR_ENABLE,ESC_MASK,&ex_status);
      if(ex_status==ESC_MASK) goto aborted;

      if(fs_sel!=_SXADDA4DCM_FSSEL_EXT)	// if not external sampling freq
	{
	      printf("sampling frequency (Hz)                             : ");
	      fs_entered=getlong((fs_sel!=_SXADDA4DCM_FSSEL_FSX)?
	      				1L:	// this is for non-C6X/FPER clock
					1L+(_C6X_FPER_SOURCE_CLK/_C6X_FPER_MAX_COUNT),
	      		         200000L,fs_entered,CR_ENABLE,ESC_MASK,&ex_status);
	      if(ex_status==ESC_MASK) goto aborted;
	}

      switch(fs_sel)	// convert Fs value to FSG or SIOX timer ticks
	  { case _SXADDA4DCM_FSSEL_FSG:	// convert Fs to FSG count
		fs = (long) (double)( 1.0e7/((double)fs_entered));
	       break;

	    case _SXADDA4DCM_FSSEL_SXTM:	// convert Fs to TM0 timer ticks
		fs = (long) (double)( ((dev_id==TP62_DEV_ID)? 25.0e6:21.0e6)/((double)fs_entered));
	       break;

            case _SXADDA4DCM_FSSEL_FSX:		// FSX mode: convert Fs to CLKX ticks
		fs = (long) (double)( 15.0e6/((double)fs_entered));
	       break;

	    default:     // nothing to do, exit
	       break;
	  }

			//----- enter A/D and D/A channels list
      if( KBD_ADC_RECORD_CMND || KBD_ADC_TO_DAC_CMND )
	  {
		if(enter_channel_list("enter A/D channel list (0-ch0, 1-ch1, 2-ch2, 3-ch3) : ",
				&ad_chan, &ad_chan_total)!=0) goto aborted;
	  }

      if( KBD_DAC_SINE_CMND)
	  {
            if(enter_channel_list("enter D/A channel list (0-ch0, 1-ch1, 2-ch2, 3-ch3) : ",
		 	       &da_chan, &da_chan_total)!=0) goto aborted;
	  }

      return(0);
}





/*****************************************************************************
// fill on-board SRAM with zero or sine waves with F0/x1.5/x2/x3 frequencies
*****************************************************************************/

#define FILL_SINE   1
#define FILL_ZERO   0

 int __fill_dac_sram(int fill_mode)
  {   long count, start, q, n, n1, j, chan_count, w_len;

      CLEAR_CURRENT_LINE;
      printf("filling SRAM ...\r");

      for (chan_count=0; chan_count<CH_TOTAL_MAX; chan_count++)
       {
	    start=MDAC_SADDR;
	    j=start+(scan_len*chan_count*dat_type);
	    count=scan_len;
	    n=0L;

	    do
	      {
		TP6X_set_smp_ptr(j,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&D);
		w_len=(TP6X_SMP_SMP_LEN-j%TP6X_SMP_SMP_LEN)/dat_type;
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++)
		  {
		    if(fill_mode==FILL_SINE)
		      {
			 switch(chan_count)
			  {
			   case 0:
			     *D++ = (long) ((double) (30000.0 *
				     sin(50.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 1:
			     *D++ = (long) ((double) (20000.0 *
				     sin(75.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			   case 2:
			     *D++ = (long) ((double) (15000.0 *
				     sin(100.0*M_PI*((double)(n1+n))/((double) scan_len))));
		    	    break;
			   case 3:
			     *D++ = (long) ((double) (10000.0 *
				     sin(150.0*M_PI*((double)(n1+n))/((double) scan_len))));
			    break;
			  }
		      }
		     else
			  *D++ = 0L;
		  }

                if(TP6X_DPRAM_ERR_DATA)
	           {
		     CLEAR_CURRENT_LINE;
                     printf("error: DPRAM error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=(q*dat_type);
		n+=q;
	      }
	     while (count>0);
         }

     CLEAR_CURRENT_LINE;
     return(0);
  }



 int __read_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    long *LPTR;

            LPTR=(long *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {
		TP6X_set_smp_ptr(j,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&D);
		w_len=(TP6X_SMP_SMP_LEN-j%TP6X_SMP_SMP_LEN)/dat_type;

                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *LPTR++= *D++);

                if(TP6X_DPRAM_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: DPRAM error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=(q*dat_type);
	      }
	     while (count>0);
       return(0);

  }


 int __write_sb_data(long sb_addr, long len, void *pc_ptr)
  {
    long j, count,n1,q, w_len;
    long *LPTR;


            LPTR=(long *)pc_ptr;
	    j=sb_addr;
	    count=len;

	    do
	      {
		TP6X_set_smp_ptr(j,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&D);
		w_len=(TP6X_SMP_SMP_LEN-j%TP6X_SMP_SMP_LEN)/dat_type;
                if(count>=w_len) q=w_len;
		    else	q=count;

                for(n1=0;n1<q;n1++, *D++=*LPTR++ );

                if(TP6X_DPRAM_ERR_DATA)
	           {
		      CLEAR_CURRENT_LINE;
                     printf("error: DPRAM error\n\n");
           	     return(1);
	           }
		count-=q;
		j+=(q*dat_type);
	      }
	     while (count>0);
       return(0);

  }


#define TMP_BUF_LEN	0x8000L


 int __move_adc_to_dac(long buf_len)
  {
    long count, j, q, jd;
    long *TMP;

      CLEAR_CURRENT_LINE;
      printf("moving SRAM data ...\r");


    if((TMP=(long *) halloc(TMP_BUF_LEN*dat_type,1))==NULL)
       { printf("error: out of memory\n\n");
	 return(1);
       }

                      /* copy ADC data to DAC buffer */

	    j=MADC_SADDR;
	    jd=MDAC_SADDR;
	    count=buf_len;

	    do
	      {
                if(count>=TMP_BUF_LEN) q=TMP_BUF_LEN;
		    else	q=count;
		if(   __read_sb_data(j,q,TMP)!=0
		   || __write_sb_data(jd,q,TMP)!=0
		  )
           	   {
                     hfree(TMP);return(1);
	           }
		count-=q;
		j+=(q*dat_type);
		jd+=(q*dat_type);
	      }
	     while (count>0);


    CLEAR_CURRENT_LINE;
    hfree(TMP);
    return(0);
  }



int do_cmd(int adc_scan_enable)
 {
  adc_scan_flag=0;	// clear "SPACE"-key pressed flag

  TP6X_set_smp_ptr(MCD_SADDR,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&MCD);

  MCD->cmnd		= cmnd;
  MCD->epoch_len	= scan_len;
  MCD->status		= MSTAT_ATTN;

  MCD->mode		= dcm_mode;
  MCD->fs_sel		= fs_sel;
  MCD->fs		= fs;
  MCD->synclk_sel	= synclk_sel;

	  // configure AD/DA channels in accordance with CMND
  if( KBD_DAC_SINE_CMND)
    {
	  MCD->ad_chan		= 0;
	  MCD->da_chan		= da_chan;
	  MCD->ad_chan_total	= 0;
	  MCD->da_chan_total	= da_chan_total;
    }

  if( KBD_ADC_RECORD_CMND)
    {
	  MCD->ad_chan		= ad_chan;
	  MCD->da_chan		= 0;
	  MCD->ad_chan_total	= ad_chan_total;
	  MCD->da_chan_total	= 0;
    }

  if(KBD_DAC_REPLAY_CMND)
    {
          MCD->ad_chan		= 0;
	  MCD->da_chan		= ad_chan;
	  MCD->ad_chan_total	= 0;
	  MCD->da_chan_total	= ad_chan_total;
    }

  if(KBD_ADC_TO_DAC_CMND)
    {
	  MCD->ad_chan		= ad_chan;
	  MCD->da_chan		= ad_chan;
	  MCD->ad_chan_total	= ad_chan_total;
	  MCD->da_chan_total	= ad_chan_total;
    }

  TP6X_SET_HM_RQ0_RG(5);  // set MASTER ATTN via interrupt HM_RQ0 and pass token #5

  if( KBD_ADC_RECORD_CMND )
	printf(" running... Press <SPACE> to start recording or <ESC> to exit to DOS...\r");
     else
  	printf(" running... Press <SPACE> to abort or <ESC> to exit to DOS...\r");

  do                      // wait for the ESC pressed or data ready
   {
     if(key_pressed())
       { switch (read_char())
	  { case KEY_ESC:
 	      exe_aborted:
	         CLEAR_CURRENT_LINE;
	         printf("<execution aborted, TP6x is reset>\n");
	      m_restart:
	         TP6X_SET_MRESET;
		 TP6X_CLR_DPRAM_ERR;
	         return(1);


            case 0x20:  // issue ADC SCAN synch
                 if (adc_scan_enable)
		  {
                    if (!adc_scan_flag)
	    	      { CLEAR_CURRENT_LINE;
		        printf("\rADC scanning...\r");
		        adc_scan_flag=MLU_ON;
		        MCD->status=MSTAT_GO;
		      }
		  }
		  else
		  {	// release ABORT command
                    if (!adc_scan_flag)
		      { CLEAR_CURRENT_LINE;
		        printf("\roperation is aborted...\r");
		        adc_scan_flag=MLU_ON;
		        MCD->status=MSTAT_ABORT;
		      }
		  }
		 break;

            default:
		 break;
	  }
       }
   }
    while(MCD->status!=MSTAT_RDY);

printf("\n");

   if (MCD->err!=MLU_OK)
     { CLEAR_CURRENT_LINE;
       printf("error (do_cmnd): %ld\n", MCD->err);
       return (1);
     }
   return(0);
 }



#define BOARD_ID	0
unsigned dev_id, index;
char dev_name[40];


main()

{  int r;
  char err_mess[100],dev_name[100];


   printf("\nDemo for TORNADO-P6x with T/SDAS-ADDA4/16Q/200K SIOX DCM. v.1.1p\n");
   printf("(C) MicroLAB Systems Ltd, 1994-2000\n\n");

   if(TP6X_open()!=TP6X_OK)
      {
	 error:
	    TP6X_last_err_message(err_mess);
            printf("err: %i  (%s)\n",TP6X_last_err(),err_mess);
	goto exit;
      }
   printf("boards installed: %i.  Selecting board #0...\n", TP6X_boards());

   TP6X_sel_board_id(BOARD_ID);
   TP6X_board_info(BOARD_ID,&dev_id,&index,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   TP6X_dev_name(dev_name);
   if(TP6X_err_flag()) goto error;

   printf("TP6x device id = %x (%s)     index=%x\n",
	   dev_id, dev_name, index);

   TP6X_CLR_DPRAM_ERR;
   TP6X_SET_SMP_MODE_SPA;
   TP6X_SET_MRESET;
   TP6X_SET_MRESET_MODE_HOST;
   TP6X_SET_DSP_AMEN_OFF;
   TP6X_HPI_INIT;
   TP6X_SET_IM_RG(0);
   TP6X_SET_PCIC_INTCSR(0);
   if(TP6X_err_flag()) goto error;

   TP6X_set_smp_ptr(MCD_SADDR,TP6X_SMP_DPRAM_AREA,TP6X_SMP_SMP0_SEL,&MCD);

   		// init TP6x and validate the response code

      MCD->m_id=0xffffffff;
      MCD->prg_id=0xffffffff;
      MCD->status=0xffffffff;
      MCD->err=0xffffffff;
      MCD->cmnd=0xffffffff;
      MCD->adc_data_saddr=0;
      MCD->dac_data_saddr=0;
      MCD->epoch_len=0;

   MLU_set_bell(1,&TB);  // set time control =1s
   TP6X_SET_MGO;

   while (MCD->status!=MSTAT_RDY) // wait for DSP will init or timeout
     {
       if(MLU_check_bell(&TB)<0)
	   { printf("error: no reply from TORNADO-P6x\n\n");
	     goto exit;
	   }
       if(TP6X_DPRAM_ERR_DATA)
	   { printf("error: DPRAM error\n\n");
	     goto exit;
	   }
     }


   if(   MCD->m_id!=T6X_M_ID    // validate MCD returned
      || MCD->prg_id!=T6X_PRG_ID
      || MCD->status!=MSTAT_RDY
      || MCD->cmnd!=0
      || MCD->err!=0
      || MCD->adc_data_saddr==0
      || MCD->dac_data_saddr==0
      || MCD->adc_data_saddr==MCD->dac_data_saddr
      || MCD->epoch_len==0
      || MCD->epoch_len>=(TP6X_DPM_DPRAM_LEN/(dat_type*2L*CH_TOTAL_MAX))
      || MCD->adc_data_saddr>(TP6X_DPM_DPRAM_LEN-(MCD->epoch_len*dat_type*CH_TOTAL_MAX))
      || MCD->dac_data_saddr>(TP6X_DPM_DPRAM_LEN-(MCD->epoch_len*dat_type*CH_TOTAL_MAX))
     )
       {
	printf("error: invalid TP6x initialization data\n");
	 goto exit;
       }


	// -- read communication params fm TORNADO
	//    and configure local environment
   MADC_SADDR = MCD->adc_data_saddr;
   MDAC_SADDR = MCD->dac_data_saddr;
   max_scan_len = scan_len = MCD->epoch_len;

   __fill_dac_sram(FILL_ZERO);


while(1)
 {
   printf("\n commands : \n");
   printf("      0 : Output F0/x1.5/x2/x3 sine waves via D/A\n");
   printf("      1 : Record A/D data in on-board SRAM\n");
   printf("      2 : Cyclic Playback recorded A/D data from on-board SRAM\n");
   printf("      3 : Continuous pass-through (A/D->D/A)\n");
   printf("    Esc : exit to DOS\n\n");

   printf("enter command: ");

kbd_loop:
   switch(kbd_cmnd=read_char())
     {
        case '0':
                CLEAR_CURRENT_LINE;
		printf("Output F0/x1.5/x2/x3 sine waves via D/A >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
		scan_len=max_scan_len;

		if (    inp_param()!=0
		     || __fill_dac_sram(FILL_SINE)!=0
                     || do_cmd(MLU_OFF)!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
                   ) goto exit;

                prev_kbd_adc_record_cmnd=0;
		break;


	case '1':

                CLEAR_CURRENT_LINE;
		printf("Record A/D data in on-board SRAM >>>>\n");
		cmnd = MCMND_DAS_DAC_SCAN;

		scan_len=max_scan_len;

		if (    inp_param()!=0
		     || __fill_dac_sram(FILL_ZERO)!=0
                     || do_cmd(MLU_ON)!=0
		     || __move_adc_to_dac(scan_len*CH_TOTAL_MAX)!=0
                   ) goto exit;

                prev_kbd_adc_record_cmnd=1;
		break;




	case '2':
			// this cmnd is allowed only in case
			// the previously released cmnd was the ADC_RECORD
			// this is used to copy prev ADC_CHAN to new DAC_CHAN
		if(prev_kbd_adc_record_cmnd==0)
		   { beep_err();
		     goto kbd_loop;
		   }

                CLEAR_CURRENT_LINE;
		printf("Playback recorded A/D data from on-board SRAM >>>>\n");

		cmnd = MCMND_DAS_DAC_SCAN;
		scan_len=max_scan_len;

                if(   inp_param()!=0
		   || do_cmd(MLU_OFF)!=0
                  ) goto exit;
		break;



	case '3':

                CLEAR_CURRENT_LINE;
		printf("Continuous Pass-through (A/D->D/A) >>>>\n");
		cmnd = MCMND_ADC_TO_DAC;
		scan_len = 0x10; // use reduced AD/DA data buffer length

		if (   inp_param()!=0
		    || do_cmd(MLU_OFF)!=0
		   ) goto exit;

                prev_kbd_adc_record_cmnd=0;
                break;


        case KEY_ESC:
	  printf(" <aborted>\n");
	  goto exit;

	default:
		beep_err();
		goto kbd_loop;
     }

 }
exit:
  printf("\n");
}