/***************************************************************
	Application specific execution module.
	DCM: T/SDAS-ADDA4/16Q/200K
****************************************************************/

/*-------- MACROS for programming timers as I/O line */

#define TM_OUT_0  0
#define TM_OUT_1  4

#define	SET_CONTROL_MODE	{*C6X_TM1_CONTROL = TM_OUT_0;}
#define	SET_DATA_MODE		{*C6X_TM1_CONTROL = TM_OUT_1;}

#define	_WAIT_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))

#define	_TDATA			(*C6X_MCBSP0_DXR)
#define	_RDATA			(*C6X_MCBSP0_DRR)

#define	_DAC_CHAN_DEFINED	(da_chan_total!=0)

		/* X/R interrupt handler parameters	*/
unsigned	dac_scan_flag		= ML_OFF,
		dac_cyclic_scan_flag	= ML_ON,
		dac_data_count		=0,
        	dac_data_count_max	=0,
		adc_scan_flag		= ML_OFF,
		adc_cyclic_scan_flag	= ML_OFF,
		adc_data_count		=0,
		adc_data_count_max	=0;

int		*adc_save_ptr		=NULL,	/* run-time pointers to ADC/DAC data */
		*adc_data_start_ptr	=NULL,
		*dac_data_start_ptr	=NULL,
	        *dac_get_ptr		=NULL;

int	        *dac_ptr0		=NULL,	// DAC/ADC channel pointers
		*dac_ptr1		=NULL,
		*dac_ptr2		=NULL,
		*dac_ptr3		=NULL,
		*adc_ptr0		=NULL,
		*adc_ptr1		=NULL,
		*adc_ptr2		=NULL,
		*adc_ptr3		=NULL;
		

int ad_chan_list[4],	// list of active A/D channels
    da_chan_list[4],	// list of active D/A channels
    ad_chan_total,
    da_chan_total;


#define CHAN_0		1	/* ch#0 mask */
#define CHAN_1		2	/* ch#0 mask  */
#define CHAN_2		4	/* ch#0 mask  */
#define CHAN_3		8	/* ch#0 mask  */


/* Timer-0 configuration.
   TM-0 is used as DCM external sampling frequency source.
*/
#define	TM_GC_DATA	0x303
#define TM_GC_GO_MASK	0xc0


int __delay(n)
{ int i;
	for(i=n;i>=0;i--);
}

#define	_delay_100us()	__delay(10000)
#define	_delay_40ms()	__delay(100000)


void init_sio_cmode()
 { int i;

	*C6X_MCBSP0_SPCR= TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA  ;
	*C6X_MCBSP0_PCR	= TSDAS_ADDA4_C6X_MCBSP_PCR_DATA_CMODE  ;

	*C6X_MCBSP0_XCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_CMODE  ;
	*C6X_MCBSP0_RCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_CMODE  ;
	
	*C6X_MCBSP0_SRGR= TSDAS_ADDA4_C6201_MCBSP_SRGR_DATA_CMODE;

	*C6X_MCBSP0_SPCR =   TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA
			   | TSDAS_ADDA4_C6X_MCBSP_SPCR_GO_MASK ;	
 }


void init_sio_dmode()
 {
   unsigned sio_pcr;

        sio_pcr =     TSDAS_ADDA4_C6X_MCBSP_PCR_DATA_DMODE
		  | (((MCD->fs_sel)!=3)     ? TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_XFSX:
		  			      TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_IFSX
                    )
		  | (((MCD->synclk_sel)==0) ? TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_XCLKX:
		  			      TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_ICLKX
                    ) ;

	*C6X_MCBSP0_SPCR= TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA  ;
	*C6X_MCBSP0_PCR	= sio_pcr;

	        // set frame length in accordance with number of channels
	*C6X_MCBSP0_XCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_DMODE  ;
	*C6X_MCBSP0_RCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_DMODE ;

                // configure CLKX generator, which will be used for external SYNCLK mode
		// configure McBSP/FPER generator for SIOX/FSX sampling frequency
		// !!!note: FPER generator can generate sampling frequency only up to CLKX(15MHz)/4096 value
	*C6X_MCBSP0_SRGR=  TSDAS_ADDA4_C6201_MCBSP_SRGR_DATA_DMODE
			 | (((MCD->fs_sel)!=3) ? 0:(((MCD->fs)<<TSDAS_ADDA4_C6X_MCBSP_SRGR_FPER_SHIFT) & TSDAS_ADDA4_C6X_MCBSP_SRGR_FPER_BITMASK )
			   )
                        ;
	
		// enable McBSP-0 operation
	*C6X_MCBSP0_SPCR =   TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA
			   | TSDAS_ADDA4_C6X_MCBSP_SPCR_GO_MASK ;	
 }




/*----------- transmitter interrupt --------------*/

int	_x_count =0;	// XMIT word counter
int	_x_sync  =0;	// XMIT SCAN mode sync flag
int 	_x_data ;	// transmitted data

interrupt void xmit_int()
{
        _x_data=0;	// assume that no valid data is transferred

        if(dac_scan_flag==ML_ON)
	  {
		//-- ensure that SCAN mode is initialized fm the WORD#0
            if((_x_sync==0) && (_x_count==1)) goto exit;

            _x_sync = 1;	// set SYNC flag
	    if(_x_count==0)
                 {	// word #0: check for active chan#0/#1
		    if(da_chan_total>0)
                      {
                        _x_data = ((dac_ptr0[dac_data_count] <<16));

			if(da_chan_total>1)
                           _x_data |= ((dac_ptr1[dac_data_count]) & 0xffff);
		      }
                 }
	       else
		 {	// word #1: check for active chan#2/#3
		    if(da_chan_total>2)
                      {
                           _x_data = ((dac_ptr2[dac_data_count] <<16));

			if(da_chan_total>3)
                           _x_data |= ((dac_ptr3[dac_data_count]) & 0xffff);
		      }

                    dac_data_count++;
	            if(dac_data_count>=dac_data_count_max)
			 { if(dac_cyclic_scan_flag==ML_ON)
				dac_data_count=0;
			      else
			        dac_scan_flag=ML_OFF;
			 }
		  }
	  }

   exit:
      _TDATA = _x_data ;	// transmit data
      _x_count++;		// increment X-words counter within 0..1
      _x_count %= 2;	
}


/*---------- receiver interrupt --------------*/

int	_r_data;
int	_r_count =0;	// RECEIVE word counter
int	_r_sync  =0;	// RCV SCAN mode sync flag

interrupt void recv_int()   /* receiver interrupt */
{

     	_r_data= _RDATA;

        if(adc_scan_flag==ML_ON)
	  {
		//-- ensure that SCAN mode is initialized fm the WORD#0
            if(_r_sync==0 && _r_count==1) goto exit;

	    _r_sync = 1;	// set SYNC flag
	    if(_r_count==0)
                 {	// word #0: active chan#0/#1
		    if(ad_chan_total>0)
                      {
                        adc_ptr0[adc_data_count] = (_r_data>>16);

			if(ad_chan_total>1)
                          adc_ptr1[adc_data_count] = ((_r_data<<16) >>16);
		      }
                 }
	       else
		 {      // word #1: active chan#2/#3
		    if(ad_chan_total>2)
                      {
                        adc_ptr2[adc_data_count] = (_r_data>>16);

			if(ad_chan_total>3)
                          adc_ptr3[adc_data_count] = ((_r_data<<16) >>16);
		      }

	            if((++adc_data_count) >=adc_data_count_max)
		      { if(adc_cyclic_scan_flag==ML_ON)
				adc_data_count=0;
			      else
			        adc_scan_flag=ML_OFF;
		      }
		  }
	  }
    exit:
       _r_count++;             // increment R-words counter within 0..1
       _r_count %= 2;

}



int sio0_transmit_receive(int c)
	// transmit/receive data in CONTROL MODE
{
   _WAIT_TRDY;	// wait for transmitter ready
   _TDATA = (c & 0xffff);

   _WAIT_RRDY;	// wait for receiver ready
   if( (_RDATA & 0xffff)==c)
   	      return(0);	// OK, data received=transmitted
   	else  return(1);	// error

}


	// scan enable flags for AD/DA channels
	// and fill in the channel lists for IRQ handlers
void  _create_adda_chan_list()
{ int i, chan_mask;

     ad_chan_total=0;
     da_chan_total=0;

     for (i=0;i<4;i++)	// clear channel list for correct pointers
       { ad_chan_list[i] = 0;
	 da_chan_list[i] = 0;
       }


     for (i=0,chan_mask=1;i<4;i++)
	{
	  if(((MCD->ad_chan) & chan_mask)!=0)
	     ad_chan_list[ad_chan_total++] = i;

	  if(((MCD->da_chan) & chan_mask)!=0)
	     da_chan_list[da_chan_total++] = i;

          chan_mask <<= 1;
	}

		// create channel pointers for R/X interrupts
     dac_ptr0	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[0])] );
     dac_ptr1	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[1])] );
     dac_ptr2	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[2])] );
     dac_ptr3	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[3])] );

     adc_ptr0	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[0])] );
     adc_ptr1	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[1])] );
     adc_ptr2	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[2])] );
     adc_ptr3   = & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[3])] );

}



int	das_dac_scan()
  {
  	int i;
	unsigned control_reg, status;


	C6X_GINT_DISABLE;

		//---- configure DCM in CONTROL mode
	SET_CONTROL_MODE;	
	init_sio_cmode();

	control_reg =   TSDAS_ADDA4_CTRL_RG_WR_CMND

		     |  ( ((MCD->synclk_sel)==0) ? TSDAS_ADDA4_CTRL_RG_CLK_SEL_ONBOARD:
		     				   TSDAS_ADDA4_CTRL_RG_CLK_SEL_SXCLKX
		     	)	// SYN-CLK selector

		     | TSDAS_ADDA4_CTRL_RG_SDW_FMT64	// data format = 64bits

		     | ( ((MCD->fs_sel)==0) ? TSDAS_ADDA4_CTRL_RG_FS_SEL_FSG:
                          ( ((MCD->fs_sel)==1) ? TSDAS_ADDA4_CTRL_RG_FS_SEL_XFS:
                              ( ((MCD->fs_sel)==2) ? TSDAS_ADDA4_CTRL_RG_FS_SEL_SXTM:
			     	                     TSDAS_ADDA4_CTRL_RG_FS_SEL_SXFSX
			      )
			  )
                       )	// Fs selector

		     | (((MCD->mode)==0) ? TSDAS_ADDA4_CTRL_RG_ASYNC_MODE:
				           TSDAS_ADDA4_CTRL_RG_SYNC_MODE
                       )		// mode selector

		     | ((((MCD->ad_chan)&0x1)==0) ? 0:TSDAS_ADDA4_CTRL_RG_ADC0_EN)		// A/D channels
		     | ((((MCD->ad_chan)&0x2)==0) ? 0:TSDAS_ADDA4_CTRL_RG_ADC1_EN)		
		     | ((((MCD->ad_chan)&0x4)==0) ? 0:TSDAS_ADDA4_CTRL_RG_ADC2_EN)		
		     | ((((MCD->ad_chan)&0x8)==0) ? 0:TSDAS_ADDA4_CTRL_RG_ADC3_EN)		

		     | ((((MCD->da_chan)&0x1)==0) ? 0:TSDAS_ADDA4_CTRL_RG_DAC0_EN)		// D/A channels
		     | ((((MCD->da_chan)&0x2)==0) ? 0:TSDAS_ADDA4_CTRL_RG_DAC1_EN)		
		     | ((((MCD->da_chan)&0x4)==0) ? 0:TSDAS_ADDA4_CTRL_RG_DAC2_EN)		
		     | ((((MCD->da_chan)&0x8)==0) ? 0:TSDAS_ADDA4_CTRL_RG_DAC3_EN)		
		;

		// load control register
         if (    sio0_transmit_receive(control_reg)!=0
	    )	
        	{ hw_err:
	                  C6X_GINT_DISABLE;
			  C6X_INT_DISABLE_ALL;
			  return(MERR_HW_ERR);
		}

	 switch(MCD->fs_sel)
	  { case 0:	// load FSG
		if(    sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG0_WR_CMND
					      | ((MCD->fs) & TSDAS_ADDA4_FSG_RG_BITMASK)
                                            )!=0
        	    || sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG1_WR_CMND
		    			      | (((MCD->fs) >>8)  & TSDAS_ADDA4_FSG_RG_BITMASK)
                                            )!=0
        	    || sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG2_WR_CMND
		    			      | (((MCD->fs) >>16)  & TSDAS_ADDA4_FSG_RG_BITMASK)
                                            )!=0
		  )	goto hw_err;

	       break;

	    case 2:	// SX-TM#0 mode: load Fs to TM0 timer
		*C6X_TM0_CONTROL = TM_GC_DATA;
		*C6X_TM0_PERIOD	 = MCD->fs	;
		*C6X_TM0_CONTROL = TM_GC_DATA | TM_GC_GO_MASK	;
	       break;

	    default:     // casee #3 and other: nothing to do, exit
	       break;
	  }



//-------- enter data mode

				/* set X/R interrupt driver parameters */
	if(MCD->cmnd == MCMND_DAS_DAC_SCAN)
	 {      	// ADC/DAC scan at full length
		dac_data_start_ptr	= DAC;
		dac_data_count		= 0;
	        dac_data_count_max	= MCD->epoch_len;

		adc_data_start_ptr	= ADC;
		adc_data_count		= 0;
        	adc_data_count_max	= MCD->epoch_len;

		dac_cyclic_scan_flag	= ML_ON;
		adc_cyclic_scan_flag	= ML_OFF;

		adc_scan_flag		= ML_OFF;
		dac_scan_flag		= ML_ON;
	}
	else
	{	      	// ADC->DAC copy with short length
		dac_scan_flag	= ML_OFF;
		adc_scan_flag	= ML_OFF;

		dac_data_start_ptr	= ADC;
	        dac_get_ptr		= ADC;
		dac_data_count		= 0;
	        dac_data_count_max	= 0x10;

		adc_data_start_ptr	= ADC;
        	adc_save_ptr		= ADC;
		adc_data_count		= 0;
	        adc_data_count_max	= 0x10;

		dac_cyclic_scan_flag	= ML_ON;
		adc_cyclic_scan_flag 	= ML_ON;

		dac_scan_flag		= ML_ON;
		adc_scan_flag		= ML_ON;
	}

               	// clear IRQ handler params
	_r_count = 0;
	_x_count = 1;
	_r_sync = 0;
	_x_sync = 0;

		// analyze input AD/DA channels and create
		// chan scan lists for IRQ handlers
	_create_adda_chan_list();	

	C6X_GINT_DISABLE;
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	init_sio_dmode();	// configure McBSP for DATA mode transmission
	C6X_INT_CLEAR(4);	// clear XMIT/RCV irq
	C6X_INT_CLEAR(5);	

	if(_DAC_CHAN_DEFINED)	
	 {  _TDATA = 0; 	// push XMIT if DAC(s) are defined
	    C6X_INT_ENABLE(4);	// enable XMIT irqs if DAC(s) are defined
	 }

	C6X_INT_ENABLE(5);      // enable RCV irq

		// enable interrupts and finally start AD/DA acquisition...
	C6X_GINT_ENABLE;
	SET_DATA_MODE;		// push DCM DATA mode


                // background host sync processing loop
	while(1)
	 { 	
		// scan and interprete MCD->status flag fm host
	   status = MCD->status ;

	   	// check for ABORT cmnd and exit immediately
           if( status==MSTAT_ABORT) break;	

                // check for MSTAT_GO cmnd (start A/D acquisition)
		// if detected:
		//     1. enable ADC SCAN
		//     2. wait until ADC scanning will OFF
		//     3. exit
	   if( status==MSTAT_GO)	
            {	adc_scan_flag	= ML_ON;
		while(adc_scan_flag!=ML_OFF);	
		break;		
	    }
	 }
		//-- exit with no error
	adc_scan_flag	= ML_OFF;
	dac_scan_flag	= ML_OFF;

	C6X_GINT_DISABLE;
	C6X_INT_DISABLE_ALL;

	return(MERR_OK);
  }

