/*****************************************************************
	T/SDAS-ADDA4/16Q/200K demo/test utility.
	(C) MicroLAB Systems Ltd, 1994-1999

******************************************************************/

#include	<math.h>
#include	<stdlib.h>


/* PC-to-DSP communication descriptor */

struct	MCD_DD
 { int 		m_id;            // Master ID
   int 		prg_id;          // Master program ID

   int 		status;          // Master STATUS: MSTAT_RDY/BUSY/ATTN
   int 		err;             // command exec error

   int 		cmnd;            // command to exec
   int 		fq;		 // unused
   unsigned 	epoch_len;       // epoch length
   unsigned 	adc_data_ptr,    // ADC data buffer start
        	dac_data_ptr;    // DAC data sptr

   unsigned 	mode;            // ASYNC (==0) or SYXN (==1) mode
   unsigned 	fs_sel;          // Fs Selector (0 - FSG, 1- Ext Fs,
   				 //               2 -SIOX TM-0, 3 - FSX (ASYNC mode only)
   unsigned 	fs;		 // Fs value (in Hz)

   unsigned 	ad_chan;         // A/D channels (bits 0..3)
   unsigned	da_chan;         // D/A channels (bits 0..3)

   unsigned 	ad_chan_total;   // total number of A/D channels
   unsigned 	da_chan_total;   // total number of D/A channels
   unsigned 	synclk_sel; 	 // synchro-clock source
};

/*	default address for MCD in DPRAM */
struct	MCD_DD *MCD	= (struct MCD_DD *)0x200;


#define	T6X_ID			0x101
#define	PRG_ID			0x100

#define	MSTAT_RDY		0 /* Master status values */
#define	MSTAT_BUSY		1
#define	MSTAT_ATTN		2
#define	MSTAT_GO		3
#define	MSTAT_ABORT		4

#define	MERR_OK			0 /* Master returned error codes */
#define	MERR_INV_CMND		1 /*     ; invalid command */
#define	MERR_INV_ATTN		2
#define	MERR_HW_ERR		3
#define	MERR_SMALL_MEMORY_ERR	4

#define	MCMND_NOP		0
#define	MCMND_ABORT		1

#define	MCMND_DAS_SCAN		0x100
#define	MCMND_DAS_DAC_SCAN	0x101
#define	MCMND_DAC_SING_LOAD	0x102
#define	MCMND_ADC_TO_DAC	0x103

#define	ML_OFF			0
#define	ML_ON			1

#define	MDATA_BUF_LEN		(0x7800L/8L) /* data words per channel */


int	 *ADC=NULL,
	 *DAC=NULL;		/* ptrs to the ADC/DAC data */

int	host_rq_flag=0;


#include	"c6x_def.h"
#include	"tp6x_dsp.h"
#include	"sxa4_dsp.h"
#include	"t6tp6_a4.i"


interrupt void host_int()   /* H->M IRQ */
{
	 host_rq_flag=1;
}

cpu_init()
 {
		// configure EMIF
	*C6X_EMIF_GCR=TP6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=TP6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=TP6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=TP6X_EMIF_CE2_DATA;

	*C6X_EMIF_SDRAM=TP6X_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TP6X_EMIF_SDRAM_TIMER_DATA;	
	*C6X_EMIF_CE3=TP6X_EMIF_CE3_DATA;

		// assign INT4    = McBSP-0 Transmitter interrupt
		// assign INT5    = McBSP-0 Receiver interrupt
		// assume INT7    = Host->DSP #0 interrupt (via EXT_INT7)
	if(   C6X_INTMUX_SET(4,C6X_INTSEL_MCBSP0_XINT)!=MERR_OK
	   || C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP0_RINT)!=MERR_OK
	   ) return(-1);

	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_POS);
	C6X_INT_DISABLE_ALL;

		// now configure external EXT_INT7 source
	TP6X_set_mirq_sel(TP6X_MIRQ0,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ1,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ2,TP6X_MIRQ_SEL_OFF);
	TP6X_set_mirq_sel(TP6X_MIRQ3,TP6X_MIRQ_SEL_HM_RQ0);
	TP6X_set_mnmi_sel(TP6X_MIRQ_SEL_OFF);

		// now remove reset for SIOX/PIOX
	TP6X_set_pxsx_run(0x7);
 }



int	cmnd_decoder()			/* command decoder	*/
 {
	if(MCD->status!=MSTAT_ATTN) return(MERR_INV_ATTN);

	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
	   case MCMND_NOP:		/* common cmnds */
	   case MCMND_ABORT:
		 return(MERR_OK);

                                        /* application specific commands */
	   case MCMND_DAS_DAC_SCAN:
	   case MCMND_ADC_TO_DAC:
	   	 return( das_dac_scan() );

	   default:			/* all unknown commands cause error */
		 return( MERR_INV_CMND );
	 }

 }



void main()
{ int r;
	r=MERR_OK;

		//-- init CPU and allocate ADC/DAC buffers fm the heap
	if(   cpu_init()!=MERR_OK
	   || (ADC=(int *) malloc(MDATA_BUF_LEN*4L*4L))==NULL
	   || (DAC=(int *) malloc(MDATA_BUF_LEN*4L*4L))==NULL
	  )
	    r=MERR_HW_ERR;

	MCD->m_id		= T6X_ID;
	MCD->prg_id		= PRG_ID;
	MCD->cmnd		= 0;
	MCD->fq			= 0;
	MCD->epoch_len		= MDATA_BUF_LEN;
	MCD->err		= r;
	MCD->adc_data_ptr	= (unsigned) ADC;
	MCD->dac_data_ptr	= (unsigned) DAC;
	MCD->status		= MSTAT_RDY;


	while(1)			/* continuous loop	*/
	 {
	   host_rq_flag=0;		/* clear HOST RQ */

	   C6X_INT_CLEAR_ALL;
	   C6X_INT_ENABLE(7); 		/* enable IRQ-3 (HM_RQ) */
	   C6X_GINT_ENABLE;		/* set GIE bit */

           while(host_rq_flag==0) ;	// wait until Host RQ

	   C6X_INT_DISABLE(7); 		/* disable HOST RQ */

	   MCD->err = cmnd_decoder();	/* call exec & return error code */
           MCD->status = MSTAT_RDY;	/* clear status flag */
	 }
}
