/********************************************************************************
	T/SDAS-ADDA4/16Q/200K DCM Stand-alone Demo/Test 
	for TORNADO-6x/P6x/E6x/P64xx/E64xx DSP Systems/Controllers. rev.1B

	(C) MicroLAB Systems Ltd, 1994-2003

	WEB:    www.mlabsys.com
	FTP:    ftp://ftp.mlabsys.com/sft
	e-mail: techsupport@mlabsys.com


	Description:
	------------
	  1. This demo runs on TORNADO-6x/P6x/P64xx DSP Systems and TORNADO-E6x/E64xx DSP 
	  	 Controllers within DSP environment only in stand-alone mode and does not require
		 host control.
	  2. This DSP demo has been designed to be user configured by means of
		 compiler run-time definitions inside the A4_C6X.CFG file, that allows to
		 select different demo modes and parameters:
			- TORNADO platform selection (TORNADO-6x, TORNADO-P6x, TORNADO-E6x, TORNADO-P64xx,
			  TORNADO-E64xx)
			- number of A/D and D/A analog signal acquisition channels involved
			- demo mode selection (either SINUS output via all DAC channels, or
			  ADC->DAC pass-through mode for each AD/DA channel)
			- run-time configuration parameters (sampling source/frequency,
			  synchronization mode, etc)
	  3. MIRAGE-510DX/P510D JTAG emulator and TI C6000 CCS tools are required to
		 recompile this demo and to upload code to the DSP environment.
	  4. It is possible to debug DSP code using JTAG emulator.
	  5. DSP on-chip McBSP0 SIO port is used for communication with
		 T/SDAS-ADDA4/16Q/200K DCM, i.e. DCM is assumed to be instaled to on-board
		 SIOX-A rev.B site.
	  6. TM1 pin of the SIOX site (i.e. DSP on-chip timer #1 output) is assumed to
		 be used as an OUTPUT pin for CONTROL/DATA mode control of T/SDAS-ADDA4/16Q/200K DCM.
		 TM0 pin of the SIOX site (i.e. DSP on-chip timer #1 output) is assumed to
		 be used as an OUTPUT pin for alternate sampling frequency source for
		 T/SDAS-ADDA4/16Q/200K DCM.
	  7. The demo performance is as it is due to non-optimized demo code.


	T/SDAS-ADDA4/16Q/200K daughter-card module installation:
	--------------------------------------------------------
	  1. T/SDAS-ADDA4/16Q/200K DCM must be installed into SIOX-A site at TORNADO mainboard.
  	  2. J1 jumper of T/SDAS-ADDA4/16Q/200K DCM must be set to 2-3 position in
		 order to enable TM1 SIOX output pin for DCM CONTROL/DATA mode selection.
  	  3. The corresponding TORNADO on-board "TM-0/1 OUT" jumpers/switches (J8/J9 for
		 TORNADO-6x, J5/J6 for TORNADO-P62/P67, SW4-3/4 for TORNADO-P6202/P6203,
		 J5/J6 for TORNADO-E62/E67, SW5-1/2 for TORNADO-E6202/E6203, SW5-1/2 for TORNADO-P64xx/E64xx)
		 shall be both set to ON in order to enable DSP on0chip timers outputs.


	Loading the Demo and Initialization:
	------------------------------------
		All demos can be loaded into DSP environment via MIRAGE JTAG emulator
		(or compatible).


  	Copyright:
  	----------
		This demo is distributed free of charge and as it is without
		any obligation from MicroLAB Systems.


	file: main .C file

*********************************************************************************/

// general includes
#include	"math.h"
#include	"stdlib.h"

//---------------------- include DEMO CNF file ---------------------------
#include	"a4_c6x.cfg"	// include run-time compiler defs for demo mode and params

	// TORNADO-E6x specific definitions and include
#if(_E6X_SELECTED)
#include	"c6x_def.h"		// include C6x/C64xx defs
#include	"te6x_dsp.h"	// include defs for E6x/E64xx
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#define _SET_SIOX_RESET		TE6X_set_pxsx_run(0)
#endif

	// TORNADO-6x specific definitions and include
#if(_T6X_SELECTED)
#include	"c6x_def.h"		// include C6x/C64xx defs
#include	"t6x_dsp.h"		// include defs for T6x
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#define _SET_SIOX_RESET		T6X_set_pxsx_run(0)

#endif

	// TORNADO-P6x specific definitions and include
#if(_TP6X_SELECTED)
#include	"c6x_def.h"		// include C6x/C64xx defs
#include	"tp6x_dsp.h"	// include defs for TP6x/E64xx
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#define _SET_SIOX_RESET		TP6X_set_pxsx_run(0)
#endif

	// TORNADO-E64xx specific definitions and include
#if(_E64XX_SELECTED)
#define	__TE6X_SELECT_E64XX__	1
#include	"c6x_def.h"		// include C6x/C64xx defs
#include	"te6x_dsp.h"	// include defs for E6x/E64xx
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x7)
#define _SET_SIOX_RESET		TE6X_set_pxsx_run(0)
#endif

	// TORNADO-P64xx specific definitions and include
#if(_TP64XX_SELECTED)
#define	__TP6X_SELECT_TP64XX__	1
#include	"c6x_def.h"		// include C6x/C64xx defs
#include	"tp6x_dsp.h"	// include defs for TP6x/TP64xx
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#define _SET_SIOX_RESET		TP6X_set_pxsx_run(0)
#endif


//-----------------------------------------------------------------------



// DSP master control descriptor (MCD)

struct	MCD_DD
 {
   int 			status;		     // Master STATUS: MSTAT_RDY/BUSY/ATTN
   int 			err;        	 // command exec error

   int 			cmnd;   	     // command to exec
   unsigned 	epoch_len;       // epoch length

   unsigned 	mode;            // DAQ mode: ASYNC-DATA (==0) or SYNC-DATA (==1)

   unsigned 	fs_sel;          // Fs Selector (0 - FSG, 1- Ext Fs, 2 -SIOX TM-0, 3 - FSX (ASYNC mode only)
   unsigned 	fs;				 // Fs value (in Hz)

   unsigned 	ad_chan;         // A/D channels (bits 0..3)
   unsigned		da_chan;         // D/A channels (bits 0..3)

   unsigned 	synclk_sel; 	 // synchro-clock source
};

//	default address for MCD in SRAM
struct	MCD_DD *MCD	= (struct MCD_DD *)0x200;

// Master status values
#define	MSTAT_RDY				0
#define	MSTAT_BUSY				1
#define	MSTAT_ATTN				2
#define	MSTAT_GO				3
#define	MSTAT_ABORT				4

// Master returned error codes
#define	MERR_OK					0
#define	MERR_INV_CMND			1 	//     ; invalid command
#define	MERR_INV_ATTN			2
#define	MERR_HW_ERR				3
#define	MERR_SMALL_MEMORY_ERR	4
#define	MERR_INV_CNF_ERR		5

// Master commands defs
#define	MCMND_NOP				0
#define	MCMND_ABORT				1

#define	MCMND_DAC_SINE_OUTPUT	0x102
#define	MCMND_ADC_TO_DAC		0x103


// general defs
#define	ML_OFF					0
#define	ML_ON					1


// maximum data array length for one ADC/DAC channel (in samples)
// ADC/DAC data bufers are allocated fm heap, which is in external memory
#define	DATA_BUF_LEN			0x0800L


// global vars
int	 *ADC=NULL,*DAC=NULL;			// ptrs to the ADC/DAC data

int	dsp_type=0;						// dsp type, which is required for proper setting of CLKX/FSX freqs

// dsp type defs
#define	_DSP_C6201				0
#define	_DSP_C6701				1
#define	_DSP_C6202				2
#define	_DSP_C6203				3
#define	_DSP_C64XX_600MHZ		4
#define	_DSP_C64XX_720MHZ		5
#define	_DSP_C64XX_1000MHZ		6

#define	_DSP_IS_C6201			(dsp_type==_DSP_C6201)
#define	_DSP_IS_C6701			(dsp_type==_DSP_C6701)
#define	_DSP_IS_C6202			(dsp_type==_DSP_C6202)
#define	_DSP_IS_C6203			(dsp_type==_DSP_C6203)
#define	_DSP_IS_C64XX_600MHZ	(dsp_type==_DSP_C64XX_600MHZ)
#define	_DSP_IS_C64XX_720MHZ	(dsp_type==_DSP_C64XX_720MHZ)
#define	_DSP_IS_C64XX_1000MHZ	(dsp_type==_DSP_C64XX_1000MHZ)




// include T/SDAS-ADDA4/16Q/200K header and application specific DSP code
#include	"sxa4_dsp.h"
#include	"a4_c6x.i"





#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
// include DAC SINE fill-in function in case DEMO FUNCTION is DAC_SINE_OUTPUT

#define	M_PI	3.141592

void fill_dac_sine()
// fill-in DAC buffers with sine data with different frequency for each channel
{
	int *D;
    unsigned  chan_count,n;
	float scan_len;

   	scan_len= (float) MCD->epoch_len;

    for (chan_count=0,D=DAC; chan_count<8; chan_count++)
       {
            for(n=0;n<DATA_BUF_LEN;n++)
			{
			 switch(chan_count)
			  {
			   case 0:
			     *D++ = (int) ((float) (30000.0*sin(10.0*M_PI*((float)n)/scan_len)));
			      break;

			   case 1:
			     *D++ = (int) ((float) (20000.0*sin(20.0*M_PI*((float)n)/scan_len)));
			     break;

			   case 2:
			     *D++ = (int) ((float) (15000.0*sin(40.0*M_PI*((float)n)/scan_len)));
		    	 break;

			   case 3:
			     *D++ = (int) ((float) (10000.0*sin(80.0*M_PI*((float)n)/scan_len)));
			     break;

			  }
		    }
       }
}
#endif




cpu_init()
// perform DSP initialization
 {
	int i;

	// set DCM CONTROL mode
	SET_CONTROL_MODE;

	// clear & disable all IRQs
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	// perform common initialization:
	// 	assign INT4    = McBSP-0 Transmitter interrupt
	// 	assign INT5    = McBSP-0 Receiver interrupt
	if(   C6X_INTMUX_SET(4,C6X_INTSEL_MCBSP0_XINT)!=MERR_OK
	   || C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP0_RINT)!=MERR_OK
	   ) return(-1);



	// perform TORNADO-E6x DSP System initialization, set the DSP type
#if(_E6X_SELECTED)

		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);

		// set the DSP type
	if(TE62_BOARD)
		dsp_type=_DSP_C6201;
	else
		if(TE67_BOARD)
			dsp_type=_DSP_C6701;
		else
			if(TE6202_BOARD)
				dsp_type=_DSP_C6202;
			else
				dsp_type=_DSP_C6203;
#endif


	// perform TORNADO-E64xx DSP System initialization, set the DSP type
#if(_E64XX_SELECTED)

		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);

		// set the DSP type upon the on-board DSP clock frequency ID
	dsp_type= ((TE64XX_DSP_CLK_FREQ_IS_600MHZ) ? _DSP_C64XX_600MHZ:
				((TE64XX_DSP_CLK_FREQ_IS_720MHZ) ? _DSP_C64XX_720MHZ:_DSP_C64XX_1000MHZ));
#endif


	// perform TORNADO-6x DSP System initialization, set DSP type
#if(_T6X_SELECTED)

		// init DSP EMIF
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;

		// set the DSP type
	if(T62_BOARD)
		dsp_type=_DSP_C6201;
	else
		dsp_type=_DSP_C6701;
#endif


	// perform TORNADO-P6x DSP System initialization, set DSP type
#if(_TP6X_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);

		// set the DSP type
	if(TP62_BOARD)
		dsp_type=_DSP_C6201;
	else
		if(TP67_BOARD)
			dsp_type=_DSP_C6701;
		else
			if(TP6202_BOARD)
				dsp_type=_DSP_C6202;
			else
				dsp_type=_DSP_C6203;
#endif

	// perform TORNADO-P64xx DSP System initialization, set DSP type
#if(_TP64XX_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif();

		// set the DSP type upon the on-board DSP clock frequency ID
	dsp_type= ((TP64XX_DSP_CLK_FREQ_IS_600MHZ) ? _DSP_C64XX_600MHZ:
				((TP64XX_DSP_CLK_FREQ_IS_720MHZ) ? _DSP_C64XX_720MHZ:_DSP_C64XX_1000MHZ));
#endif


	// apply/release SIOX-A reset
	_SET_SIOX_RESET;		// set SIOX reset in order to reset the DCM h/w
	for(i=0;i<10000;i++);	// wait a little bit
	_REMOVE_SIOX_RESET;		// release SIOX reset

	return(0);
 }




int	cmnd_decoder()
// decodes command
 {
	MCD->status=MSTAT_BUSY;

	switch(MCD->cmnd)
	 {
       	// application specific commands
	   case MCMND_DAC_SINE_OUTPUT:
	   case MCMND_ADC_TO_DAC:
	   	 return( das_dac_scan() );

		// all unknown commands cause error
	   default:			
		 return( MERR_INV_CMND );
	 }
 }






void main()
{ 
	int r=MERR_OK;	// declare R as rurned error code for error tracking



	// first, disable IRQs if this is the program restart under emultor 
	// without DSP reset
	C6X_GINT_DISABLE;

	// init CPU and allocate ADC/DAC buffers fm the heap
	if(   cpu_init()!=MERR_OK
	   || (ADC=(int *) malloc(DATA_BUF_LEN*4L*4L))==NULL
	   || (DAC=(int *) malloc(DATA_BUF_LEN*4L*4L))==NULL
	  )
    {
		r=MERR_HW_ERR;
		goto error;
	}

	// set MCD params from the .CFG file
	MCD->epoch_len		= DATA_BUF_LEN;		// default AD/DA bufer length to '0'
	MCD->status			= MSTAT_RDY;
	MCD->ad_chan		= _ADC_CHANNELS_;	// default AD/DA channels to NONE
	MCD->da_chan		= _DAC_CHANNELS_;

	MCD->cmnd			= _FUNC_;
	MCD->mode			= _DAQ_MODE_;
	MCD->fs_sel			= _FS_SEL_;
	MCD->fs				= _FS_;
	MCD->synclk_sel		= _SYNC_CLK_SEL_;


	// fill-in DAC data buffers with SINE data for DAC SINE OUTPUT demo function
#if(_FUNC_DAC_SINE_OUTPUT_SELECTED_)
	fill_dac_sine();
#endif

	MCD->err = cmnd_decoder();	// call exec & return error code
    MCD->status = MSTAT_RDY;	// clear status flag

	// get here on any error condition
error:
	while(1);
}
