/***************************************************************
	Application specific execution module.
	DCM: T/SDAS-ADDA4/16Q/200K
	host TONRADO platform: T6x/E6x/TP6x/E64xx/TP64xx
****************************************************************/

// MACROS for programming timers as I/O line 
#define TM_OUT_0  0
#define TM_OUT_1  4

#define	SET_CONTROL_MODE	{*C6X_TM1_CONTROL = TM_OUT_0;}
#define	SET_DATA_MODE		{*C6X_TM1_CONTROL = TM_OUT_1;}


// McBSP port access macros
#define	_WAIT_TRDY			while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_RRDY			while(!((*C6X_MCBSP0_SPCR) & 0x2))

#define	_TDATA				(*C6X_MCBSP0_DXR)
#define	_RDATA				(*C6X_MCBSP0_DRR)

// AD/DA channels analysis macros
#define	_DAC_CHAN_DEFINED	(da_chan_total!=0)
#define	_ADC_CHAN_DEFINED	(ad_chan_total!=0)


// X/R interrupt handler parameters
unsigned	dac_scan_flag			= ML_OFF,
			dac_cyclic_scan_flag	= ML_ON,
			dac_data_count			=0,
        	dac_data_count_max		=0,
			adc_scan_flag			= ML_OFF,
			adc_cyclic_scan_flag	= ML_OFF,
			adc_data_count			=0,
			adc_data_count_max		=0;

int			*adc_save_ptr			=NULL,	// run-time pointers to ADC/DAC data
			*adc_data_start_ptr		=NULL,
			*dac_data_start_ptr		=NULL,
	        *dac_get_ptr			=NULL;

int	        *dac_ptr0				=NULL,	// DAC/ADC channel pointers
			*dac_ptr1				=NULL,
			*dac_ptr2				=NULL,
			*dac_ptr3				=NULL,
			*adc_ptr0				=NULL,
			*adc_ptr1				=NULL,
			*adc_ptr2				=NULL,
			*adc_ptr3				=NULL;
		

int 		ad_chan_list[4],				// list of active A/D channels
    		da_chan_list[4],				// list of active D/A channels
    		ad_chan_total			=0,
    		da_chan_total			=0;


// Timer-0 configuration.
// TM-0 is used as DCM external sampling frequency source.
#define	TM_GC_DATA		0x303
#define TM_GC_GO_MASK	0xc0





void __delay(n)
{ int i;
	for(i=n;i>=0;i--);
}

// looped delays defs for different DSP types
// (C64xx are averagely x3 times faster than C62xx/C67xx)
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define	_wait_for_mcbsp_setup()	__delay(3000)
	#define	_delay_100us()			__delay(30000)
	#define	_delay_40ms()			__delay(300000)
#else
	#define	_wait_for_mcbsp_setup()	__delay(1000)
	#define	_delay_100us()			__delay(10000)
	#define	_delay_40ms()			__delay(100000)
#endif

void init_sio_cmode()
 { 
	*C6X_MCBSP0_SPCR= TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA  ;
	*C6X_MCBSP0_PCR	= TSDAS_ADDA4_C6X_MCBSP_PCR_DATA_CMODE  ;

	*C6X_MCBSP0_XCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_CMODE  ;
	*C6X_MCBSP0_RCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_CMODE  ;

	// set SRGR upon the DSP type in order to obtain predefined CLKX frequency value
	*C6X_MCBSP0_SRGR= ((_DSP_IS_C6201)? TSDAS_ADDA4_C6201_MCBSP_SRGR_DATA_CMODE:
							((_DSP_IS_C6701)? TSDAS_ADDA4_C6701_MCBSP_SRGR_DATA_CMODE:
						  		((_DSP_IS_C6202)? TSDAS_ADDA4_C6202_MCBSP_SRGR_DATA_CMODE:
							  		((_DSP_IS_C6203)? TSDAS_ADDA4_C6203_MCBSP_SRGR_DATA_CMODE:
								  		((_DSP_IS_C64XX_600MHZ)? TSDAS_ADDA4_C64XX_600MHZ_MCBSP_SRGR_DATA_CMODE:
									  		((_DSP_IS_C64XX_720MHZ)? TSDAS_ADDA4_C64XX_720MHZ_MCBSP_SRGR_DATA_CMODE:
									  									TSDAS_ADDA4_C64XX_1000MHZ_MCBSP_SRGR_DATA_CMODE
								    		)
							    		)
						    		)
					    		)
					  		)
 			  		  )
					   ;

	// wait some time for McBSP will enter into synchronization
	_wait_for_mcbsp_setup();

	*C6X_MCBSP0_SPCR =   TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA
					   | TSDAS_ADDA4_C6X_MCBSP_SPCR_GO_MASK ;

	// wait some time for McBSP will enter into synchronization
	_wait_for_mcbsp_setup();
 }


void init_sio_dmode()
 {
   unsigned sio_pcr;

	// set PCR upon internal/external FSX/CLKX
    sio_pcr =     TSDAS_ADDA4_C6X_MCBSP_PCR_DATA_DMODE
			  | (((MCD->fs_sel)!=_FS_SEL_SIOX_FSX_) ?
						TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_XFSX:
							TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_IFSX
                )
			  | (((MCD->synclk_sel)==_SYNC_CLK_SEL_INTERNAL_) ?
						TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_XCLKX:
						    TSDAS_ADDA4_C6X_MCBSP_PCR_DMODE_ICLKX
                )
			;

	*C6X_MCBSP0_SPCR= TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA  ;
	*C6X_MCBSP0_PCR	= sio_pcr;

    // set frame length in accordance with number of channels
	*C6X_MCBSP0_XCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_DMODE  ;
	*C6X_MCBSP0_RCR = TSDAS_ADDA4_C6X_MCBSP_XRCR_DATA_DMODE ;

    // configure CLKX generator, which will be used for external SYNCLK mode
	// configure McBSP/FPER generator for SIOX/FSX sampling frequency
	// !!!note: FPER generator can generate sampling frequency only up to CLKX(15MHz)/4096 value
	*C6X_MCBSP0_SRGR=  ((_DSP_IS_C6201)? TSDAS_ADDA4_C6201_MCBSP_SRGR_DATA_DMODE:
							((_DSP_IS_C6701)? TSDAS_ADDA4_C6701_MCBSP_SRGR_DATA_DMODE:
						  		((_DSP_IS_C6202)? TSDAS_ADDA4_C6202_MCBSP_SRGR_DATA_DMODE:
							  		((_DSP_IS_C6203)? TSDAS_ADDA4_C6203_MCBSP_SRGR_DATA_DMODE:
								  		((_DSP_IS_C64XX_600MHZ)? TSDAS_ADDA4_C64XX_600MHZ_MCBSP_SRGR_DATA_DMODE:
								  			((_DSP_IS_C64XX_720MHZ)? TSDAS_ADDA4_C64XX_720MHZ_MCBSP_SRGR_DATA_DMODE:
																		TSDAS_ADDA4_C64XX_1000MHZ_MCBSP_SRGR_DATA_DMODE
											)
							  			)
						  			)
					  			)
							)
					   )
					 | (((MCD->fs_sel)!=_FS_SEL_SIOX_FSX_) ? 0:
								 (    ((TSDAS_ADDA4_CLKX_FREQ/(MCD->fs))  <<TSDAS_ADDA4_C6X_MCBSP_SRGR_FPER_SHIFT)
									& TSDAS_ADDA4_C6X_MCBSP_SRGR_FPER_BITMASK
								 )
					   )
                        ;

	// wait some time for McBSP will enter into synchronization
	_wait_for_mcbsp_setup();

	// enable McBSP-0 operation
	*C6X_MCBSP0_SPCR =   TSDAS_ADDA4_C6X_MCBSP_SPCR_DATA
					   | TSDAS_ADDA4_C6X_MCBSP_SPCR_GO_MASK ;

	// wait some time for McBSP will enter into synchronization
	_wait_for_mcbsp_setup();
 }




//----------- transmitter interrupt --------------

int	_x_count =0;	// XMIT word counter
int	_x_sync  =0;	// XMIT SCAN mode sync flag
int _x_data ;	// transmitted data

interrupt void xmit_int()
{
      _x_data=0;	// assume that no valid data is transferred

      if(dac_scan_flag==ML_ON)
	  {
		//-- ensure that SCAN mode is initialized fm the WORD#0
            if((_x_sync==0) && (_x_count==1)) goto exit;

        _x_sync = 1;	// set SYNC flag
	    if(_x_count==0)
        {	// word #0: check for active chan#0/#1
		    if(da_chan_total>0)
            {
              	_x_data = ((dac_ptr0[dac_data_count] <<16));

				if(da_chan_total>1)
                    _x_data |= ((dac_ptr1[dac_data_count]) & 0xffff);
			}
        }
	    else
		{	// word #1: check for active chan#2/#3
		    if(da_chan_total>2)
            {
				_x_data = ((dac_ptr2[dac_data_count] <<16));

				if(da_chan_total>3)
    				_x_data |= ((dac_ptr3[dac_data_count]) & 0xffff);
			}

            dac_data_count++;

			if(dac_data_count>=dac_data_count_max)
			 {
				if(dac_cyclic_scan_flag==ML_ON)
					dac_data_count=0;
			      else
			        dac_scan_flag=ML_OFF;
			 }
		}
	  }

   exit:
      _TDATA = _x_data ;	// transmit data
      _x_count++;			// increment X-words counter within 0..1
      _x_count %= 2;
}




//---------- receiver interrupt --------------

int	_r_data;
int	_r_count =0;	// RECEIVE word counter
int	_r_sync  =0;	// RCV SCAN mode sync flag

interrupt void recv_int()
{

	  _r_data= _RDATA;

	  if(adc_scan_flag==ML_ON)
	  {
		//-- ensure that SCAN mode is initialized fm the WORD#0
		if(_r_sync==0 && _r_count==1) goto exit;

	    _r_sync = 1;	// set SYNC flag
	    if(_r_count==0)
        {	// word #0: active chan#0/#1
		    if(ad_chan_total>0)
			{
				adc_ptr0[adc_data_count] = (_r_data>>16);

				if(ad_chan_total>1)
                    adc_ptr1[adc_data_count] = ((_r_data<<16) >>16);
			}
        }
	    else
		{      // word #1: active chan#2/#3
		    if(ad_chan_total>2)
            {
                adc_ptr2[adc_data_count] = (_r_data>>16);
		
				if(ad_chan_total>3)
                    adc_ptr3[adc_data_count] = ((_r_data<<16) >>16);
		    }

            if((++adc_data_count) >=adc_data_count_max)
			{ 
				if(adc_cyclic_scan_flag==ML_ON)
					adc_data_count=0;
			      else
			        adc_scan_flag=ML_OFF;
		    }
		}
	  }

    exit:
       _r_count++;             // increment R-words counter within 0..1
       _r_count %= 2;

}



int sio0_transmit_receive(int c)
// transmit/receive data in CONTROL MODE
{
	_WAIT_TRDY;	// wait for transmitter ready
   	_TDATA = (c & 0xffff);

   	_WAIT_RRDY;	// wait for receiver ready
   	if( (_RDATA & 0xffff)==c)
   		return(0);	// OK, data received=transmitted
	else  
		return(1);	// error

}


// scan enable flags for AD/DA channels
// and fill in the channel lists for IRQ handlers
void  _create_adda_chan_list()
{ int i, chan_mask;

     ad_chan_total=0;
     da_chan_total=0;

     for (i=0;i<4;i++)	// clear channel list for correct pointers
     {
		ad_chan_list[i] = 0;
		da_chan_list[i] = 0;
     }


     for (i=0,chan_mask=1;i<4;i++)
	 {
	 	if(((MCD->ad_chan) & chan_mask)!=0)
	    	 ad_chan_list[ad_chan_total++] = i;

	  	if(((MCD->da_chan) & chan_mask)!=0)
	    	 da_chan_list[da_chan_total++] = i;

        chan_mask <<= 1;
	 }

	 // create channel pointers for R/X interrupts
     dac_ptr0	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[0])] );
     dac_ptr1	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[1])] );
     dac_ptr2	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[2])] );
     dac_ptr3	= & (dac_data_start_ptr[dac_data_count_max * (da_chan_list[3])] );

     adc_ptr0	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[0])] );
     adc_ptr1	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[1])] );
     adc_ptr2	= & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[2])] );
     adc_ptr3   = & (adc_data_start_ptr[adc_data_count_max * (ad_chan_list[3])] );

}





int	das_dac_scan()
// controls and configures DAQ procedure
{
	unsigned control_reg;

	// analyze input cnofiguration params in order to exclude confusions
	if(	   ((MCD->mode==_DAQ_MODE_SYNC_) && (MCD->fs_sel==_FS_SEL_SIOX_FSX_))
		|| ((MCD->cmnd==MCMND_DAC_SINE_OUTPUT) && ((MCD->da_chan & _DAC_ALL_EN_MASK_)==0))
		|| ((MCD->cmnd==MCMND_ADC_TO_DAC) && ((MCD->ad_chan & _ADC_ALL_EN_MASK_)==0))
	  )
		return(MERR_INV_CNF_ERR);



	// correct enable flags for ADC/DAC channels upon the demo function
	if(MCD->cmnd == MCMND_DAC_SINE_OUTPUT)
	{
		// DAC sine output
		// disable all A/D channels for this demo function
		MCD->ad_chan			= 0;
	}
	else
	{
		// ADC->DAC copy
		// set DAC channels equal to ADC channels for this demo function
		MCD->da_chan			= MCD->ad_chan;
	}


	// disable DSP IRQs
	C6X_GINT_DISABLE;

	// configure DCM in CONTROL mode
	SET_CONTROL_MODE;
	init_sio_cmode();


	// assemble CONTROL REGISTER word
	control_reg =   TSDAS_ADDA4_CTRL_RG_WR_CMND

			     |  MCD->synclk_sel					// SYN-CLK selector

			     | 	TSDAS_ADDA4_CTRL_RG_SDW_FMT64	// assume 64bit data word format for this demo

			     | 	MCD->fs_sel						// Fs selector

			     | 	MCD->mode						// mode selector

			     | (MCD->ad_chan & _ADC_ALL_EN_MASK_)		// A/D channels
			     | ((MCD->da_chan & _DAC_ALL_EN_MASK_)<<4)	// D/A channels
			;

	// load control register
    if (sio0_transmit_receive(control_reg)!=0)
   	{
		hw_err:
              C6X_GINT_DISABLE;
			  C6X_INT_DISABLE_ALL;
			  return(MERR_HW_ERR);
	}

	// now analyze sampling frequency selector and load FSG or DSP timer if required
	switch(MCD->fs_sel)
	{
			// calculate and load FSG
		case _FS_SEL_FSG_:
			if(    sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG0_WR_CMND
									      | ((TSDAS_ADDA4_FSG_CLKIN_FREQ/(MCD->fs)) & TSDAS_ADDA4_FSG_RG_BITMASK)
                                        )!=0
        	    || sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG1_WR_CMND
		    						      | (((TSDAS_ADDA4_FSG_CLKIN_FREQ/(MCD->fs)) >>8)  & TSDAS_ADDA4_FSG_RG_BITMASK)
                                        )!=0
        	    || sio0_transmit_receive(   TSDAS_ADDA4_FSG_RG2_WR_CMND
			    			      		  | (((TSDAS_ADDA4_FSG_CLKIN_FREQ/(MCD->fs)) >>16)  & TSDAS_ADDA4_FSG_RG_BITMASK)
                                        )!=0
		  	  )
				goto hw_err;

	        break;


			// SX-TM#0 mode: load Fs to DSP on-chip TM0 timer upon the DSP type
	    case _FS_SEL_SIOX_TM_:
			*C6X_TM0_CONTROL = TM_GC_DATA;

				// 	Note the /2 final divider for C6x TM period due to the final :2  for duty=50%.
				// 	C62xx/C67xx DSP provide the DSP_CLK/4 as a source clock to the on-chip timers,
				//	whereas C64xx DSPs provide the DSP_CLK/8 timer source clock.
			*C6X_TM0_PERIOD	 = 	((_DSP_IS_C6201)? (200000000L/(4*MCD->fs)):
									((_DSP_IS_C6701)? (167000000L/(4*MCD->fs)):
								  		((_DSP_IS_C6202)? (250000000L/(4*MCD->fs)):
									  		((_DSP_IS_C6203)? (300000000L/(4*MCD->fs)):
									  			((_DSP_IS_C64XX_600MHZ)? (600000000L/(8*MCD->fs)):
										  			((_DSP_IS_C64XX_720MHZ)? (720000000L/(8*MCD->fs)):
										  			  							(1000000000L/(8*MCD->fs))
											  		)
										  		)
									  		)
								  		)
									)
							   	)/2
							   ;

			*C6X_TM0_CONTROL = TM_GC_DATA | TM_GC_GO_MASK	;
	       	break;


			// get here in case of Fs selector is FSX. Nothing to do, since
			// McBSP FSX counter will be programmed during McBSP initialization
	    default:
	       break;
	  }


	// set X/R interrupt driver parameters
	if(MCD->cmnd == MCMND_DAC_SINE_OUTPUT)
	 {
		// DAC sine output at full length
		dac_data_start_ptr		= DAC;
		dac_data_count			= 0;
	    dac_data_count_max		= MCD->epoch_len;

		adc_data_start_ptr		= ADC;
		adc_data_count			= 0;
      	adc_data_count_max		= MCD->epoch_len;

		dac_cyclic_scan_flag	= ML_ON;
		adc_cyclic_scan_flag	= ML_OFF;

		adc_scan_flag			= ML_OFF;
		dac_scan_flag			= ML_ON;
	}
	else
	{
		// ADC->DAC copy with short length
		dac_scan_flag			= ML_OFF;
		adc_scan_flag			= ML_OFF;

		dac_data_start_ptr		= ADC;
	    dac_get_ptr				= ADC;
		dac_data_count			= 0;
	    dac_data_count_max		= 0x10;

		adc_data_start_ptr		= ADC;
        adc_save_ptr			= ADC;
		adc_data_count			= 0;
	    adc_data_count_max		= 0x10;

		dac_cyclic_scan_flag	= ML_ON;
		adc_cyclic_scan_flag 	= ML_ON;

		dac_scan_flag			= ML_ON;
		adc_scan_flag			= ML_ON;
	}

	// analyze input AD/DA channels and create chan scan lists for IRQ handlers
	_create_adda_chan_list();

   	// reset X/R IRQ handlers params
	_r_count = 0;
	_x_count = 1;
	_r_sync = 0;
	_x_sync = 0;

	C6X_GINT_DISABLE;
	C6X_INT_DISABLE_ALL;
	C6X_INT_CLEAR_ALL;

	// configure McBSP for DATA mode transmission
	init_sio_dmode();

	// clear XMIT/RCV irq
	C6X_INT_CLEAR(4);
	C6X_INT_CLEAR(5);

	// push XMIT
	_TDATA = 0;

	// wait some time for McBSP will enter into synchronization
	_wait_for_mcbsp_setup();

	// enable XMIT irqs
    C6X_INT_ENABLE(4);

	// enable RCV irq if ADC channels are defined
	if(_ADC_CHAN_DEFINED)
		C6X_INT_ENABLE(5);

	// enable interrupts and finally start AD/DA acquisition...
	C6X_GINT_ENABLE;
	SET_DATA_MODE;

	// loop forever during DAQ
	while(1);
}

